/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No 
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all 
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM 
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES 
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS 
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of 
* this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2014-2016 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : r_dac_rx_if.h
* Description  : Functions for using DAC on RX devices.
************************************************************************************************************************
* History : DD.MM.YYYY Version Description
*           31.03.2014 2.00    New API
*           05.09.2014 2.10    Added support for RX64M.
*           20.01.2015 2.20    Added support for RX113.
*           02.03.2015 2.30    Added support for RX71M.
*           10.06.2015 2.40    Added support for RX231.
*           30.09.2015 2.50    Added support for RX23T.
*           01.10.2015 2.60    Added support for RX130.
*           01.12.2015 2.70    Added support for RX230.
*           01.02.2016 2.80    Added support for RX24T.
*           01.03.2016 2.90    Added support for RX65N.
*           31.08.2016 2.91    Added support for RX65N Amp not use.
*           19.12.2016 3.00    Added support for RX24U.
***********************************************************************************************************************/
#ifndef DAC_RX_IF_H
#define DAC_RX_IF_H

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "platform.h"
#include "r_dac_rx_config.h"

/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/
/* Version Number of API. */
#define DAC_VERSION_MAJOR  (3)
#define DAC_VERSION_MINOR  (00)

#if defined(BSP_MCU_RX23T) || defined(BSP_MCU_RX24T)
#define DAC_CFG_NUM_CH  (1)
#else
#define DAC_CFG_NUM_CH  (2)
#endif

/*****************************************************************************
Typedef definitions
******************************************************************************/

#if defined(BSP_MCU_RX23T) || defined(BSP_MCU_RX24T)
typedef enum e_dac_ch           // DAC channel numbers
{
    DAC_CH0    = 0,
    DAC_NUM_CH
} dac_ch_t;
#else
typedef enum e_dac_ch           // DAC channel numbers
{
    DAC_CH0    = 0,
    DAC_CH1    = 1,
    DAC_NUM_CH = 2
} dac_ch_t;
#endif


/* DAC OPEN() PARAMETER */

typedef enum e_dac_refv         // DAC reference voltage
{
    DAC_REFV_AVCC0_AVSS0    = 1,
    DAC_REFV_INTERNAL_AVSS0 = 3,
    DAC_REFV_VREFH_VREFL    = 6
} dac_refv_t;

#if defined(BSP_MCU_RX111) || defined(BSP_MCU_RX23T) || defined(BSP_MCU_RX24T)
typedef struct st_dac_cfg
{
    bool        fmt_flush_right;
} dac_cfg_t;

#elif defined(BSP_MCU_RX113) || defined(BSP_MCU_RX231) || defined(BSP_MCU_RX230)
typedef struct st_dac_cfg
{
    dac_refv_t ref_voltage;
    bool fmt_flush_right;
    bool sync_with_adc;
} dac_cfg_t;

#elif defined(BSP_MCU_RX210)
typedef struct st_dac_cfg
{
    bool        fmt_flush_right;
    bool        ch_conv_off_when_output_off;
} dac_cfg_t;

#elif defined(BSP_MCU_RX64M) || defined(BSP_MCU_RX71M) || defined(BSP_MCU_RX65N)
typedef struct st_dac_cfg
{
    bool        fmt_flush_right;
    bool        sync_with_adc;
    uint8_t     sync_unit;                      // 0 or 1
    bool        ch_conv_off_when_output_off;    // applies to both channels
} dac_cfg_t;

#elif defined(BSP_MCU_RX130) || defined(BSP_MCU_RX24U)
typedef struct st_dac_cfg
{
    bool        fmt_flush_right;
    bool        sync_with_adc;
} dac_cfg_t;

#else // RX63N/631
typedef struct st_dac_cfg
{
    bool        fmt_flush_right;
    bool        sync_with_adc;
    bool        ch_conv_off_when_output_off;
} dac_cfg_t;
#endif


/* DAC CONTROL() COMMANDS */
typedef enum e_dac_cmd
{
    DAC_CMD_OUTPUT_ON,          // Analog output of channel is enabled.
    DAC_CMD_OUTPUT_OFF,         // Analog output of channel is disabled.
#if defined(BSP_MCU_RX64_ALL) || defined(BSP_MCU_RX71M)
    DAC_CMD_AMP_ON,             // Gain of 1 amplifier. See Electrical
    DAC_CMD_AMP_OFF,            // Characteristics in HW User's Manual
#endif
    DAC_CMD_END_ENUM
} dac_cmd_t;


/* DAC API ERROR CODES */
typedef enum e_dac_err
{
    DAC_SUCCESS=0,
    DAC_ERR_BAD_CHAN,           // non-existent channel number
    DAC_ERR_INVALID_CMD,        // non-existent operation command
    DAC_ERR_INVALID_ARG,        // argument is not valid for parameter
    DAC_ERR_NULL_PTR,           // received null ptr; missing required argument
    DAC_ERR_LOCK_FAILED,        // failed to lock DAC module
    DAC_ERR_UNLOCK_FAILED,      // failed to unlock DAC module
    DAC_ERR_ADC_NOT_POWERED,    // cannot sync because ADC is not powered
    DAC_ERR_ADC_CONVERTING      // cannot sync because ADC is converting
} dac_err_t;


/*****************************************************************************
Public Functions
******************************************************************************/
dac_err_t R_DAC_Open(dac_cfg_t *p_config);
                    
dac_err_t R_DAC_Write(uint8_t const chan, uint16_t data);

dac_err_t R_DAC_Control(uint8_t const chan, dac_cmd_t const cmd);

dac_err_t R_DAC_Close(void);

uint32_t  R_DAC_GetVersion(void);

                                    
#endif /* DAC_RX_IF_H */

