/***********************************************************************/
/*                                                                     */
/*  FILE        : Main.c                                               */
/*  DATE        :Tue, Oct 31, 2006                                     */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/
/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
//#include "typedefine.h"
#include "platform.h"
#include "r_dac_rx_config.h"
#include "r_dac_rx_if.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define SINE_RESOL       16     /* resolution of sine wave */
#define UPDATE_TIMING  0x95     /* timing of updating DAC output */

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/
void main(void);
void tmr_init(void);

uint8_t sine_wave_tbl_index;

const uint16_t sine_wave_tbl[SINE_RESOL] =
{
    512,            /* (0x400 >> 1) * (1 + sine(0*360/SINE_RESOL)) */
    708,            /* (0x400 >> 1) * (1 + sine(1*360/SINE_RESOL)) */
    874,            /* (0x400 >> 1) * (1 + sine(2*360/SINE_RESOL)) */
    985,            /* (0x400 >> 1) * (1 + sine(3*360/SINE_RESOL)) */
    1023,           /* (0x400 >> 1) * (1 + sine(4*360/SINE_RESOL)) */
    985,            /* (0x400 >> 1) * (1 + sine(5*360/SINE_RESOL)) */
    874,            /* (0x400 >> 1) * (1 + sine(6*360/SINE_RESOL)) */
    708,            /* (0x400 >> 1) * (1 + sine(7*360/SINE_RESOL)) */
    512,            /* (0x400 >> 1) * (1 + sine(8*360/SINE_RESOL)) */
    316,            /* (0x400 >> 1) * (1 + sine(9*360/SINE_RESOL)) */
    150,            /* (0x400 >> 1) * (1 + sine(10*360/SINE_RESOL)) */
    39,             /* (0x400 >> 1) * (1 + sine(11*360/SINE_RESOL)) */
    0,              /* (0x400 >> 1) * (1 + sine(12*360/SINE_RESOL)) */
    39,             /* (0x400 >> 1) * (1 + sine(13*360/SINE_RESOL)) */
    150,            /* (0x400 >> 1) * (1 + sine(14*360/SINE_RESOL)) */
    316             /* (0x400 >> 1) * (1 + sine(15*360/SINE_RESOL)) */
};

/*******************************************************************************
* Function Name: main
* Description  : Main program
* Arguments    : none
* Return Value : none
*******************************************************************************/
void main(void)
{
    dac_cfg_t config;
    dac_err_t err;

    sine_wave_tbl_index = 0;

    /* Initialize ADC */
    config.fmt_flush_right = true;
    config.sync_with_adc = false;
    config.ch_conv_off_when_output_off = true;
    err = R_DAC_Open(&config);

    while (DAC_SUCCESS != err)
    {
        /* Error - loop here for debug */
    }

    err = R_DAC_Write(DAC_CH1, sine_wave_tbl[sine_wave_tbl_index]);

    while (DAC_SUCCESS != err)
    {
        /* Error - loop here for debug */
    }

    sine_wave_tbl_index++;

    err = R_DAC_Control(DAC_CH1, DAC_CMD_OUTPUT_ON);

    while (DAC_SUCCESS != err)
    {
        /* Error - loop here for debug */
    }

    tmr_init();

    while(1);
}

/*******************************************************************************
* Function Name: tmr_init
* Description  : Initialize TMR
* Arguments    : none
* Return Value : none
*******************************************************************************/
void tmr_init(void)
{
    SYSTEM.PRCR.WORD = 0xA502;
    MSTP(TMR0) = 0;                 /* Release from module stop state. */
    SYSTEM.PRCR.WORD = 0xA500;

    TMR0.TCR.BYTE = 0x48;           /* Clear TCNT by compare match A */
                                    /* enable CMIA0 */
    TMR0.TCCR.BYTE = 0x09;          /* Clock source is PCLK/2 */
    TMR0.TCNT = 0x00;
    TMR0.TCORA = UPDATE_TIMING;

    IR(TMR0,CMIA0) = 0;
    IPR(TMR0,CMIA0) = 1;
    IEN(TMR0,CMIA0) = 1;            /* interrupt enable */
}

/*******************************************************************************
* Function Name: int_CMIA0
* Description  : Interrupt CMIA0
* Arguments    : none
* Return Value : none
*******************************************************************************/
#pragma interrupt INT_TMR0_CMIA0(vect=VECT(TMR0,CMIA0))
void INT_TMR0_CMIA0(void)
{
    dac_err_t err;

    err = R_DAC_Write(DAC_CH1, sine_wave_tbl[sine_wave_tbl_index]);

    while (DAC_SUCCESS != err)
    {
        /* Error - loop here for debug */
    }

    /* setting next DAC output */
    if ( (SINE_RESOL-1) > sine_wave_tbl_index )
    {
        sine_wave_tbl_index++;
    }
    else
    {
        sine_wave_tbl_index = 0;
    }
}

